package jazzlib;

public class Adler32 implements Checksum {
	private static final int BASE = 65521;
	private int checksum;

	public Adler32() {
		this.reset();
	}

	public void reset() {
		this.checksum = 1;
	}

	public void update(final int n) {
		final int n2 = this.checksum & 0xFFFF;
		final int n3 = this.checksum >>> 16;
		final int n4 = (n2 + (n & 0xFF)) % 65521;
		this.checksum = ((n4 + n3) % 65521 << 16) + n4;
	}

	public void update(final byte[] array) {
		this.update(array, 0, array.length);
	}

	public void update(final byte[] array, int n, int i) {
		int n2 = this.checksum & 0xFFFF;
		int n3 = this.checksum >>> 16;
		while (i > 0) {
			int n4 = 3800;
			if (n4 > i) {
				n4 = i;
			}
			i -= n4;
			while (--n4 >= 0) {
				n2 += (array[n++] & 0xFF);
				n3 += n2;
			}
			n2 %= 65521;
			n3 %= 65521;
		}
		this.checksum = (n3 << 16 | n2);
	}

	public long getValue() {
		return (long) this.checksum & 0xFFFFFFFFL;
	}
}
