package jazzlib;

public class CRC32 implements Checksum {
	private int crc;
	private static int[] crc_table;

	public CRC32() {
		this.crc = 0;
	}

	private static int[] make_crc_table() {
		final int[] array = new int[256];
		for (int i = 0; i < 256; ++i) {
			int n = i;
			int n2 = 8;
			while (--n2 >= 0) {
				if ((n & 0x1) != 0x0) {
					n = (0xEDB88320 ^ n >>> 1);
				} else {
					n >>>= 1;
				}
			}
			array[i] = n;
		}
		return array;
	}

	public long getValue() {
		return (long) this.crc & 0xFFFFFFFFL;
	}

	public void reset() {
		this.crc = 0;
	}

	public void update(final int n) {
		final int n2 = ~this.crc;
		this.crc = ~(CRC32.crc_table[(n2 ^ n) & 0xFF] ^ n2 >>> 8);
	}

	public void update(final byte[] array, int n, int n2) {
		int n3 = ~this.crc;
		while (--n2 >= 0) {
			n3 = (CRC32.crc_table[(n3 ^ array[n++]) & 0xFF] ^ n3 >>> 8);
		}
		this.crc = ~n3;
	}

	public void update(final byte[] array) {
		this.update(array, 0, array.length);
	}

	static {
		CRC32.crc_table = make_crc_table();
	}
}
