package jazzlib;

class InflaterDynHeader {
	private static final int LNUM = 0;
	private static final int DNUM = 1;
	private static final int BLNUM = 2;
	private static final int BLLENS = 3;
	private static final int LENS = 4;
	private static final int REPS = 5;
	private static final int[] repMin;
	private static final int[] repBits;
	private byte[] blLens;
	private byte[] litdistLens;
	private InflaterHuffmanTree blTree;
	private int mode;
	private int lnum;
	private int dnum;
	private int blnum;
	private int num;
	private int repSymbol;
	private byte lastLen;
	private int ptr;
	private static final int[] BL_ORDER;

	public InflaterDynHeader() {
	}

	public boolean decode(final StreamManipulator streamManipulator) throws DataFormatException {
		while (true) {
			switch (this.mode) {
			case 0: {
				int lnum;
				if (streamManipulator.bits_in_buffer < 5) {
					if (streamManipulator.window_start == streamManipulator.window_end) {
						lnum = -1;
					} else {
						final int buffer = streamManipulator.buffer;
						final int n = (streamManipulator.window[streamManipulator.window_start++] & 0xFF)
								| (streamManipulator.window[streamManipulator.window_start++] & 0xFF) << 8;
						final int bits_in_buffer = streamManipulator.bits_in_buffer + 16;
						streamManipulator.bits_in_buffer = bits_in_buffer;
						final int buffer2 = buffer | n << bits_in_buffer - 16;
						streamManipulator.buffer = buffer2;
						lnum = (buffer2 & 0x1F);
					}
				} else {
					lnum = (streamManipulator.buffer & 0x1F);
				}
				this.lnum = lnum;
				if (this.lnum < 0) {
					return false;
				}
				this.lnum += 257;
				streamManipulator.buffer >>>= 5;
				streamManipulator.bits_in_buffer -= 5;
				this.mode = 1;
			}
			case 1: {
				int dnum;
				if (streamManipulator.bits_in_buffer < 5) {
					if (streamManipulator.window_start == streamManipulator.window_end) {
						dnum = -1;
					} else {
						final int buffer3 = streamManipulator.buffer;
						final int n2 = (streamManipulator.window[streamManipulator.window_start++] & 0xFF)
								| (streamManipulator.window[streamManipulator.window_start++] & 0xFF) << 8;
						final int bits_in_buffer2 = streamManipulator.bits_in_buffer + 16;
						streamManipulator.bits_in_buffer = bits_in_buffer2;
						final int buffer4 = buffer3 | n2 << bits_in_buffer2 - 16;
						streamManipulator.buffer = buffer4;
						dnum = (buffer4 & 0x1F);
					}
				} else {
					dnum = (streamManipulator.buffer & 0x1F);
				}
				this.dnum = dnum;
				if (this.dnum < 0) {
					return false;
				}
				++this.dnum;
				streamManipulator.buffer >>>= 5;
				streamManipulator.bits_in_buffer -= 5;
				this.num = this.lnum + this.dnum;
				this.litdistLens = new byte[this.num];
				this.mode = 2;
			}
			case 2: {
				int blnum;
				if (streamManipulator.bits_in_buffer < 4) {
					if (streamManipulator.window_start == streamManipulator.window_end) {
						blnum = -1;
					} else {
						final int buffer5 = streamManipulator.buffer;
						final int n3 = (streamManipulator.window[streamManipulator.window_start++] & 0xFF)
								| (streamManipulator.window[streamManipulator.window_start++] & 0xFF) << 8;
						final int bits_in_buffer3 = streamManipulator.bits_in_buffer + 16;
						streamManipulator.bits_in_buffer = bits_in_buffer3;
						final int buffer6 = buffer5 | n3 << bits_in_buffer3 - 16;
						streamManipulator.buffer = buffer6;
						blnum = (buffer6 & 0xF);
					}
				} else {
					blnum = (streamManipulator.buffer & 0xF);
				}
				this.blnum = blnum;
				if (this.blnum < 0) {
					return false;
				}
				this.blnum += 4;
				streamManipulator.buffer >>>= 4;
				streamManipulator.bits_in_buffer -= 4;
				this.blLens = new byte[19];
				this.ptr = 0;
				this.mode = 3;
			}
			case 3: {
				while (this.ptr < this.blnum) {
					int n4;
					if (streamManipulator.bits_in_buffer < 3) {
						if (streamManipulator.window_start == streamManipulator.window_end) {
							n4 = -1;
						} else {
							final int buffer7 = streamManipulator.buffer;
							final int n5 = (streamManipulator.window[streamManipulator.window_start++] & 0xFF)
									| (streamManipulator.window[streamManipulator.window_start++] & 0xFF) << 8;
							final int bits_in_buffer4 = streamManipulator.bits_in_buffer + 16;
							streamManipulator.bits_in_buffer = bits_in_buffer4;
							final int buffer8 = buffer7 | n5 << bits_in_buffer4 - 16;
							streamManipulator.buffer = buffer8;
							n4 = (buffer8 & 0x7);
						}
					} else {
						n4 = (streamManipulator.buffer & 0x7);
					}
					final int n6 = n4;
					if (n6 < 0) {
						return false;
					}
					streamManipulator.buffer >>>= 3;
					streamManipulator.bits_in_buffer -= 3;
					this.blLens[InflaterDynHeader.BL_ORDER[this.ptr]] = (byte) n6;
					++this.ptr;
				}
				this.blTree = new InflaterHuffmanTree(this.blLens);
				this.blLens = null;
				this.ptr = 0;
				this.mode = 4;
			}
			case 4: {
				int symbol;
				while (((symbol = this.blTree.getSymbol(streamManipulator)) & 0xFFFFFFF0) == 0x0) {
					this.litdistLens[this.ptr++] = (this.lastLen = (byte) symbol);
					if (this.ptr == this.num) {
						return true;
					}
				}
				if (symbol < 0) {
					return false;
				}
				if (symbol >= 17) {
					this.lastLen = 0;
				} else if (this.ptr == 0) {
					throw new DataFormatException();
				}
				this.repSymbol = symbol - 16;
				this.mode = 5;
			}
			case 5: {
				final int n7 = InflaterDynHeader.repBits[this.repSymbol];
				int n8;
				if (streamManipulator.bits_in_buffer < n7) {
					if (streamManipulator.window_start == streamManipulator.window_end) {
						n8 = -1;
					} else {
						final int buffer9 = streamManipulator.buffer;
						final int n9 = (streamManipulator.window[streamManipulator.window_start++] & 0xFF)
								| (streamManipulator.window[streamManipulator.window_start++] & 0xFF) << 8;
						final int bits_in_buffer5 = streamManipulator.bits_in_buffer + 16;
						streamManipulator.bits_in_buffer = bits_in_buffer5;
						final int buffer10 = buffer9 | n9 << bits_in_buffer5 - 16;
						streamManipulator.buffer = buffer10;
						n8 = (buffer10 & (1 << n7) - 1);
					}
				} else {
					n8 = (streamManipulator.buffer & (1 << n7) - 1);
				}
				final int n10 = n8;
				if (n10 < 0) {
					return false;
				}
				streamManipulator.buffer >>>= n7;
				streamManipulator.bits_in_buffer -= n7;
				int n11 = n10 + InflaterDynHeader.repMin[this.repSymbol];
				if (this.ptr + n11 > this.num) {
					throw new DataFormatException();
				}
				while (n11-- > 0) {
					this.litdistLens[this.ptr++] = this.lastLen;
				}
				if (this.ptr == this.num) {
					return true;
				}
				this.mode = 4;
				continue;
			}
			default: {
				continue;
			}
			}
		}
	}

	public InflaterHuffmanTree buildLitLenTree() throws DataFormatException {
		final byte[] array = new byte[this.lnum];
		System.arraycopy(this.litdistLens, 0, array, 0, this.lnum);
		return new InflaterHuffmanTree(array);
	}

	public InflaterHuffmanTree buildDistTree() throws DataFormatException {
		final byte[] array = new byte[this.dnum];
		System.arraycopy(this.litdistLens, this.lnum, array, 0, this.dnum);
		return new InflaterHuffmanTree(array);
	}

	static {
		repMin = new int[] { 3, 3, 11 };
		repBits = new int[] { 2, 3, 7 };
		BL_ORDER = new int[] { 16, 17, 18, 0, 8, 7, 9, 6, 10, 5, 11, 4, 12, 3, 13, 2, 14, 1, 15 };
	}
}
