package jazzlib;

public class InflaterHuffmanTree {
	private static final int MAX_BITLEN = 15;
	private short[] tree;
	public static InflaterHuffmanTree defLitLenTree;
	public static InflaterHuffmanTree defDistTree;
	private static final String bit4Reverse = "\u0000\b\u0004\f\u0002\n\u0006\u000e\u0001\t\u0005\r\u0003\u000b\u0007\u000f";

	public InflaterHuffmanTree(final byte[] array) throws DataFormatException {
		this.buildTree(array);
	}

	private void buildTree(final byte[] array) throws DataFormatException {
		final int[] array2 = new int[16];
		final int[] array3 = new int[16];
		for (int i = 0; i < array.length; ++i) {
			final byte b = array[i];
			if (b > 0) {
				final int[] array4 = array2;
				final byte b2 = b;
				++array4[b2];
			}
		}
		int n = 0;
		int n2 = 512;
		for (int j = 1; j <= 15; ++j) {
			array3[j] = n;
			n += array2[j] << 16 - j;
			if (j >= 10) {
				n2 += (n & 0x1FF80) - (array3[j] & 0x1FF80) >> 16 - j;
			}
		}
		if (n != 65536) {
			throw new DataFormatException("Code lengths don't add up properly.");
		}
		this.tree = new short[n2];
		int n3 = 512;
		for (int k = 15; k >= 10; --k) {
			final int n4 = n & 0x1FF80;
			n -= array2[k] << 16 - k;
			for (int l = n & 0x1FF80; l < n4; l += 128) {
				this.tree[bitReverse(l)] = (short) (-n3 << 4 | k);
				n3 += 1 << k - 9;
			}
		}
		for (int n5 = 0; n5 < array.length; ++n5) {
			final byte b3 = array[n5];
			if (b3 != 0) {
				final int n6 = array3[b3];
				int bitReverse = bitReverse(n6);
				if (b3 <= 9) {
					do {
						this.tree[bitReverse] = (short) (n5 << 4 | b3);
						bitReverse += 1 << b3;
					} while (bitReverse < 512);
				} else {
					final short n7 = this.tree[bitReverse & 0x1FF];
					final int n8 = 1 << (n7 & 0xF);
					final int n9 = -(n7 >> 4);
					do {
						this.tree[n9 | bitReverse >> 9] = (short) (n5 << 4 | b3);
						bitReverse += 1 << b3;
					} while (bitReverse < n8);
				}
				array3[b3] = n6 + (1 << 16 - b3);
			}
		}
	}

	public int getSymbol(final StreamManipulator streamManipulator) throws DataFormatException {
		int n2;
		int n;
		if (streamManipulator.bits_in_buffer < 9) {
			if (streamManipulator.window_start == streamManipulator.window_end) {
				n = (n2 = -1);
			} else {
				final int buffer = streamManipulator.buffer;
				final int n3 = (streamManipulator.window[streamManipulator.window_start++] & 0xFF)
						| (streamManipulator.window[streamManipulator.window_start++] & 0xFF) << 8;
				final int bits_in_buffer = streamManipulator.bits_in_buffer + 16;
				streamManipulator.bits_in_buffer = bits_in_buffer;
				final int buffer2 = buffer | n3 << bits_in_buffer - 16;
				streamManipulator.buffer = buffer2;
				n = (n2 = (buffer2 & 0x1FF));
			}
		} else {
			n = (n2 = (streamManipulator.buffer & 0x1FF));
		}
		final int n4 = n2;
		if (n >= 0) {
			final short n5;
			if ((n5 = this.tree[n4]) >= 0) {
				streamManipulator.buffer >>>= (n5 & 0xF);
				streamManipulator.bits_in_buffer -= (n5 & 0xF);
				return n5 >> 4;
			}
			final int n6 = -(n5 >> 4);
			final int n7 = n5 & 0xF;
			int n9;
			int n8;
			if (streamManipulator.bits_in_buffer < n7) {
				if (streamManipulator.window_start == streamManipulator.window_end) {
					n8 = (n9 = -1);
				} else {
					final int buffer3 = streamManipulator.buffer;
					final int n10 = (streamManipulator.window[streamManipulator.window_start++] & 0xFF)
							| (streamManipulator.window[streamManipulator.window_start++] & 0xFF) << 8;
					final int bits_in_buffer2 = streamManipulator.bits_in_buffer + 16;
					streamManipulator.bits_in_buffer = bits_in_buffer2;
					final int buffer4 = buffer3 | n10 << bits_in_buffer2 - 16;
					streamManipulator.buffer = buffer4;
					n8 = (n9 = (buffer4 & (1 << n7) - 1));
				}
			} else {
				n8 = (n9 = (streamManipulator.buffer & (1 << n7) - 1));
			}
			final int n11 = n9;
			if (n8 >= 0) {
				final short n12 = this.tree[n6 | n11 >> 9];
				streamManipulator.buffer >>>= (n12 & 0xF);
				streamManipulator.bits_in_buffer -= (n12 & 0xF);
				return n12 >> 4;
			}
			final int availableBits = streamManipulator.getAvailableBits();
			int n13;
			if (streamManipulator.bits_in_buffer < availableBits) {
				if (streamManipulator.window_start == streamManipulator.window_end) {
					n13 = -1;
				} else {
					final int buffer5 = streamManipulator.buffer;
					final int n14 = (streamManipulator.window[streamManipulator.window_start++] & 0xFF)
							| (streamManipulator.window[streamManipulator.window_start++] & 0xFF) << 8;
					final int bits_in_buffer3 = streamManipulator.bits_in_buffer + 16;
					streamManipulator.bits_in_buffer = bits_in_buffer3;
					final int buffer6 = buffer5 | n14 << bits_in_buffer3 - 16;
					streamManipulator.buffer = buffer6;
					n13 = (buffer6 & (1 << availableBits) - 1);
				}
			} else {
				n13 = (streamManipulator.buffer & (1 << availableBits) - 1);
			}
			final short n15 = this.tree[n6 | n13 >> 9];
			if ((n15 & 0xF) <= availableBits) {
				streamManipulator.buffer >>>= (n15 & 0xF);
				streamManipulator.bits_in_buffer -= (n15 & 0xF);
				return n15 >> 4;
			}
			return -1;
		} else {
			final int availableBits2 = streamManipulator.getAvailableBits();
			int n16;
			if (streamManipulator.bits_in_buffer < availableBits2) {
				if (streamManipulator.window_start == streamManipulator.window_end) {
					n16 = -1;
				} else {
					final int buffer7 = streamManipulator.buffer;
					final int n17 = (streamManipulator.window[streamManipulator.window_start++] & 0xFF)
							| (streamManipulator.window[streamManipulator.window_start++] & 0xFF) << 8;
					final int bits_in_buffer4 = streamManipulator.bits_in_buffer + 16;
					streamManipulator.bits_in_buffer = bits_in_buffer4;
					final int buffer8 = buffer7 | n17 << bits_in_buffer4 - 16;
					streamManipulator.buffer = buffer8;
					n16 = (buffer8 & (1 << availableBits2) - 1);
				}
			} else {
				n16 = (streamManipulator.buffer & (1 << availableBits2) - 1);
			}
			final short n18 = this.tree[n16];
			if (n18 >= 0 && (n18 & 0xF) <= availableBits2) {
				streamManipulator.buffer >>>= (n18 & 0xF);
				streamManipulator.bits_in_buffer -= (n18 & 0xF);
				return n18 >> 4;
			}
			return -1;
		}
	}

	static short bitReverse(final int n) {
		return (short) ("\u0000\b\u0004\f\u0002\n\u0006\u000e\u0001\t\u0005\r\u0003\u000b\u0007\u000f"
				.charAt(n & 0xF) << 12
				| "\u0000\b\u0004\f\u0002\n\u0006\u000e\u0001\t\u0005\r\u0003\u000b\u0007\u000f"
						.charAt(n >> 4 & 0xF) << 8
				| "\u0000\b\u0004\f\u0002\n\u0006\u000e\u0001\t\u0005\r\u0003\u000b\u0007\u000f"
						.charAt(n >> 8 & 0xF) << 4
				| "\u0000\b\u0004\f\u0002\n\u0006\u000e\u0001\t\u0005\r\u0003\u000b\u0007\u000f".charAt(n >> 12));
	}

	static {
		try {
			byte[] array;
			int i;
			for (array = new byte[288], i = 0; i < 144; array[i++] = 8) {
			}
			while (i < 256) {
				array[i++] = 9;
			}
			while (i < 280) {
				array[i++] = 7;
			}
			while (i < 288) {
				array[i++] = 8;
			}
			InflaterHuffmanTree.defLitLenTree = new InflaterHuffmanTree(array);
			final byte[] array2 = new byte[32];
			for (int j = 0; j < 32; array2[j++] = 5) {
			}
			InflaterHuffmanTree.defDistTree = new InflaterHuffmanTree(array2);
		} catch (DataFormatException ex) {
		}
	}
}
