package jazzlib;

class OutputWindow {
	public int WINDOW_SIZE;
	public int WINDOW_MASK;
	public byte[] window;
	public int window_end;
	public int window_filled;

	OutputWindow() {
		this.WINDOW_SIZE = 32768;
		this.WINDOW_MASK = this.WINDOW_SIZE - 1;
		this.window = null;
		this.window_end = 0;
		this.window_filled = 0;
	}

	public void write(final int n) {
		if (this.window_filled++ == this.WINDOW_SIZE) {
			throw new IllegalStateException("Window full");
		}
		this.window[this.window_end++] = (byte) n;
		this.window_end &= this.WINDOW_MASK;
	}

	public final void slowRepeat(int n, int n2, final int n3) {
		while (n2-- > 0) {
			this.window[this.window_end++] = this.window[n++];
			this.window_end &= this.WINDOW_MASK;
			n &= this.WINDOW_MASK;
		}
	}

	public int copyStored(final StreamManipulator streamManipulator, int min) {
		min = Math.min(Math.min(min, this.WINDOW_SIZE - this.window_filled), streamManipulator.getAvailableBytes());
		final int n = this.WINDOW_SIZE - this.window_end;
		int n2;
		if (min > n) {
			n2 = streamManipulator.copyBytes(this.window, this.window_end, n);
			if (n2 == n) {
				n2 += streamManipulator.copyBytes(this.window, 0, min - n);
			}
		} else {
			n2 = streamManipulator.copyBytes(this.window, this.window_end, min);
		}
		this.window_end = (this.window_end + n2 & this.WINDOW_MASK);
		this.window_filled += n2;
		return n2;
	}

	public void copyDict(final byte[] array, int n, int window_SIZE) {
		if (this.window_filled > 0) {
			throw new IllegalStateException();
		}
		if (window_SIZE > this.WINDOW_SIZE) {
			n += window_SIZE - this.WINDOW_SIZE;
			window_SIZE = this.WINDOW_SIZE;
		}
		System.arraycopy(array, n, this.window, 0, window_SIZE);
		this.window_end = (window_SIZE & this.WINDOW_MASK);
	}

	public int getFreeSpace() {
		return this.WINDOW_SIZE - this.window_filled;
	}

	public int getAvailable() {
		return this.window_filled;
	}

	public int copyOutput(final byte[] array, int n, int window_filled) {
		int window_end = this.window_end;
		if (window_filled > this.window_filled) {
			window_filled = this.window_filled;
		} else {
			window_end = (this.window_end - this.window_filled + window_filled & this.WINDOW_MASK);
		}
		final int n2 = window_filled;
		final int n3 = window_filled - window_end;
		if (n3 > 0) {
			System.arraycopy(this.window, this.WINDOW_SIZE - n3, array, n, n3);
			n += n3;
			window_filled = window_end;
		}
		System.arraycopy(this.window, window_end - window_filled, array, n, window_filled);
		this.window_filled -= n2;
		if (this.window_filled < 0) {
			throw new IllegalStateException();
		}
		return n2;
	}

	public void reset() {
		final boolean b = false;
		this.window_end = (b ? 1 : 0);
		this.window_filled = (b ? 1 : 0);
	}
}
