package jazzlib;

class StreamManipulator {
	public byte[] window;
	public int window_start;
	public int window_end;
	public int buffer;
	public int bits_in_buffer;

	public final int getBits(final int n) {
		int n2;
		if (this.bits_in_buffer < n) {
			if (this.window_start == this.window_end) {
				n2 = -1;
			} else {
				final int buffer = this.buffer;
				final int n3 = (this.window[this.window_start++] & 0xFF)
						| (this.window[this.window_start++] & 0xFF) << 8;
				final int bits_in_buffer = this.bits_in_buffer + 16;
				this.bits_in_buffer = bits_in_buffer;
				final int buffer2 = buffer | n3 << bits_in_buffer - 16;
				this.buffer = buffer2;
				n2 = (buffer2 & (1 << n) - 1);
			}
		} else {
			n2 = (this.buffer & (1 << n) - 1);
		}
		final int n4 = n2;
		if (n4 >= 0) {
			this.buffer >>>= n;
		}
		this.bits_in_buffer -= n;
		return n4;
	}

	public final int getAvailableBits() {
		return this.bits_in_buffer;
	}

	public final int getAvailableBytes() {
		return this.window_end - this.window_start + (this.bits_in_buffer >> 3);
	}

	public void skipToByteBoundary() {
		this.buffer >>= (this.bits_in_buffer & 0x7);
		this.bits_in_buffer &= 0xFFFFFFF8;
	}

	public final boolean needsInput() {
		return this.window_start == this.window_end;
	}

	public int copyBytes(final byte[] array, int n, int n2) {
		if (n2 < 0) {
			throw new IllegalArgumentException("length negative");
		}
		if ((this.bits_in_buffer & 0x7) != 0x0) {
			throw new IllegalStateException("Bit buffer is not aligned!");
		}
		int n3;
		for (n3 = 0; this.bits_in_buffer > 0 && n2 > 0; --n2, ++n3) {
			array[n++] = (byte) this.buffer;
			this.buffer >>>= 8;
			this.bits_in_buffer -= 8;
		}
		if (n2 == 0) {
			return n3;
		}
		final int n4 = this.window_end - this.window_start;
		if (n2 > n4) {
			n2 = n4;
		}
		System.arraycopy(this.window, this.window_start, array, n, n2);
		this.window_start += n2;
		if ((this.window_start - this.window_end & 0x1) != 0x0) {
			this.buffer = (this.window[this.window_start++] & 0xFF);
			this.bits_in_buffer = 8;
		}
		return n3 + n2;
	}

	public StreamManipulator() {
		this.window_start = 0;
		this.window_end = 0;
		this.buffer = 0;
		this.bits_in_buffer = 0;
	}

	public void reset() {
		final boolean b = false;
		this.bits_in_buffer = (b ? 1 : 0);
		this.buffer = (b ? 1 : 0);
		this.window_end = (b ? 1 : 0);
		this.window_start = (b ? 1 : 0);
	}

	public void setInput(final byte[] window, int window_start, final int n) {
		if (this.window_start < this.window_end) {
			throw new IllegalStateException("Old input was not completely processed");
		}
		final int window_end = window_start + n;
		if (0 > window_start || window_start > window_end || window_end > window.length) {
			throw new ArrayIndexOutOfBoundsException();
		}
		if ((n & 0x1) != 0x0) {
			this.buffer |= (window[window_start++] & 0xFF) << this.bits_in_buffer;
			this.bits_in_buffer += 8;
		}
		this.window = window;
		this.window_start = window_start;
		this.window_end = window_end;
	}
}
